/*
 *
 * ConfigManager
 *
 * This class is the central configuration file manager for all other classes and acts as a utility class for other classes to use.
 * It is typical to define this class as a general object (not static instance) for each class in single-threaded actions. If you need
 * a special function or action from this class; it is recommended to add it to this class and not make your own.
 *
 *
*/

package jesse.keeblarcraft.ConfigMgr;

import java.io.FileWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import com.google.common.base.Charsets;
import com.google.common.io.Files;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonIOException;
import com.google.gson.JsonSyntaxException;

import java.util.List;

import org.apache.commons.io.FileUtils;

import java.util.ArrayList;

// Import all custom exceptions
import jesse.keeblarcraft.Utils.CustomExceptions.*;

public class ConfigManager {

    // Pedantic empty constructor
    public ConfigManager() {}

    // CreateFile
    //
    // Returns true if file is created, will return false if file cannot be created (returns true if already exists)
    public Boolean CreateFile(String fileName) throws FILE_CREATE_EXCEPTION {
        Boolean ret = false;
        File file = new File(fileName);

        // Check 1: Does the file already exist?
        ret = file.exists();
        System.out.println("\033[34m Does file exist? \033[0m" + (ret ? "\033[33m YES \033[0m" : "\033[31m NO \033[0m"));

        // Check 2: If the file does not exist, attempt to create it
        if (ret == false) {
            try {
                ret = file.createNewFile();
            } catch (IOException e) {
                // The file could not be created
                throw new FILE_CREATE_EXCEPTION();
            }
        } else {
            ret = true; // This might be a hot fix, but technically the file already exists so would this be true or false? --?this statement is wild?
            System.out.println("\033[34m File (name: \033[0m" + "\033[33m" + fileName + "\033[0m" + "\033[34m) was determined to already exist. Exiting out\033[0m");
        }
        return ret;
    }

    // DeleteFile
    //
    // Returns true if file is deleted, false if could not be deleted or does not exist
    public Boolean DeleteFile(String fileName) throws FILE_DELETE_EXCEPTION {
        Boolean ret = false;
        File file = new File(fileName);

        // Step 1: Does file exist?
        ret = file.exists();

        // Step 2: If file exists, attempt to delete
        if (ret == true) {
            try {
                ret = file.delete();
            } catch (SecurityException e) {
                throw new FILE_DELETE_EXCEPTION();
            }
        } else {
            System.out.println("\033[34m Cannot delete file \033[0m" + "\033[33m" + fileName + "\033[0m" + "\033[31m because file does not exist \033[0m");
        }
        return ret;
    }

    // WriteToFile
    //
    // Will write or append to file (valid modes: "w" or "a") if file is available. Returns false if not
    public Boolean WriteToFile(String fileName, String data, String mode) {
        Boolean ret = false;

        FileWriter file;
        try {
            file = new FileWriter(fileName);
            switch(mode) {
                case "w":
                    file.write(data);
                    ret = true;
                    break;
                case "a":
                    file.append(data);
                    ret = true;
                    break;
                default:
                    System.out.println("\033[31m Invalid mode to WriteToFile! \033[0m");
                    break;
            }

            file.close();
        } catch (IOException e) {
            System.out.println("\033[31m Could not open file \033[0m" + "\033[33m" + fileName + "\033[0m" + "\033[31m to write to it! Possibly permission issue? \033[0m");
        }

        return ret;
    }


    // WriteToJsonFile
    //
    // Will write to or append to a json file. It will search if the key exists first & update the field;
    // or add a new entry. It should be noted that json objects *can* be buried inside each other. It is
    // considered best (and only) practice to call the "GetJsonStringFromFile" function first from this
    // class and simply iterate to what you would need and then update the entire entry alongside the
    // top-level key.
    //
    // NOTE: THIS DOES NOT SAFE UPDATE THE KEY OBJECT. PRE-EXISTING DATA WILL BE DELETED FOREVER
    public void WriteToJsonFile(String fileName, Object data) throws FILE_WRITE_EXCEPTION {
        Gson gson = new GsonBuilder().setPrettyPrinting().create();
        try {
            FileWriter writer = new FileWriter(fileName);
            gson.toJson(data, writer);
            writer.flush();
            writer.close();
        } catch (JsonIOException | IOException e) {
            System.out.println("\033[31m Could not successfully write to json file \033[0m");
            throw new FILE_WRITE_EXCEPTION();
        }
    }

    // GetJsonStringFromFile
    //
    // Retrieves json file and converts to desired object. Returns JsonSyntaxException if file could not be fitted to class input
    public <T> T GetJsonObjectFromFile(String fileName, Class<T> classToConvertTo) throws JsonSyntaxException {
        Gson gson = new Gson();
        String ret = "";

        // hot fix: Not sure how to return "false" for invalid conversion when I'm forced to convert or just catch... Look into a better
        // return value in the future - but for now throw JsonSyntaxException no matter what exception is caught
        try {
            File file = new File(fileName);
            ret = FileUtils.readFileToString(file, "\033[32m UTF-8 \033[0m");
        } catch (NullPointerException e) {
            System.out.println("\033[31m nullptr exception \033[0m");
            throw new JsonSyntaxException("");
        } catch (FileNotFoundException e) {
            System.out.println("\033[31m file not found \033[0m");
            throw new JsonSyntaxException("");
        } catch (java.nio.charset.UnsupportedCharsetException e) {
            System.out.println("\033[31m charset issue \033[0m");
            throw new JsonSyntaxException("");
        } catch (IOException e) {
            System.out.println("\033[31m io exception \033[0m");
            throw new JsonSyntaxException("");
        }

        return gson.fromJson(ret, classToConvertTo);
    }

    public Boolean DoesFileExist(String fileName) {
        Boolean ret = false;

        return ret;
    }

    public Boolean DoesDirectoryExist(String dirName) {
        Boolean ret = false;

        return ret;
    }

    public Boolean CreateDirectory(String dirName) throws DIRECTORY_CREATE_EXCEPTION {
        Boolean ret = false;

        File dir = new File(dirName);

        try {
            if ( ! dir.exists()) {
                ret = dir.mkdirs();
            }
        } catch (Exception e) {
            System.out.println("\033[31m Failed to make directory with name: \033[0m" + "\033[33m" + dirName + "\033[0m");
            ret = true; /// TODO: Hack to make Setup fn be fine with prev-created files! Make Setup more robust!
            throw new DIRECTORY_CREATE_EXCEPTION();
        }
        return ret;
    }

    public Boolean DeleteDirectory(String dirName) throws DIRECTORY_DELETE_EXCEPTION {
        Boolean ret = false;

        File dir = new File(dirName);

        try {
            ret = dir.delete();
            System.out.println("\033[32m Deleted directory \033[0m" + "\033[33m" + dirName + "\033[0m");
        } catch (Exception e) {
            System.out.println("\033[31m Failed to delete directory: \033[0m" + "\033[33m" + dirName + "\033[0m");
            throw new DIRECTORY_DELETE_EXCEPTION();
        }

        return ret;
    }

    // AddToKey
    //
    // Adds new text to a key if found inside the config
    public String AddToKey(String key, String newInfo) {
        String ret = "";

        return ret;
    }

    // GetFile
    //
    // Returns a file as an arraylist of all the lines in the file. Generally only used for testing
    //
    // NOTE: Returns UTF-8 Encoding of file
    public List<String> GetFile(String fileName) {
        List<String> ret = new ArrayList<String>();

        try {
            return Files.readLines(new File(fileName), Charsets.UTF_8);
        } catch (IOException e) {
            ret.clear();
        }

        return ret;
    }
}
